function [recons] = rec_asm(hol, pitch, wlen, rec_dist, zero_pad, direction)
%REC_ASM Angular Spectrum Method implementation
%
%   Inputs:
%    hol               - input hologram to reconstruct
%    pitch             - pixel pitch in meters
%    wlen              - wavelength in meters.
%    rec_dist          - reconstruction distance in meters
%    zero_pad          - zero pad the original hologram before
%                        reconstruction
%    direction         - reconstruction direction. It should be one of
%                        the following char. arrays: forward (propagation
%                        towards the object plane) or inverse (propagation 
%                        towards the hologram plane)
%   Output:
%    recons            - reconstructed field (complex magnitude)
%
%-------------------------------------------------------------------------
% Copyright(c) 2019
% University of Cagliari
% Department of Electrical and Electronic Engineering
% Italy
% All Rights Reserved.
%-------------------------------------------------------------------------                                        
%                                                                      
% The University of Cagliari - Department of Electrical and Electronic
% Engineering hereby grants to ISO/IEC JTC1 SC29 WG1
% (JPEG Committee) and each Member of ISO/IEC JTC1 SC29 WG1 (JPEG
% Committee) who participate in the Working Group dedicated to the
% standardization of JPEG Pleno, a non-exclusive, nontransferable,  
% worldwide, license under "University of Cagliari - Department of
% Electrical and Electronic Engineering" copyrights
% in this software to reproduce, distribute, display, perform and 
% create derivative works for the sole and exclusive purposes of 
% creating a hologram reconstruction software in the frame of
% the JPEG Pleno standard.
%
% Modifications to this code shall be clearly indicated and      
% identified by the relevant copyright notice(s) of the party   
% generating these changes and/or derivative works.            
%   
% Nothing contained in this software shall, except as herein 
% expressly provided, be construed as conferring by implication,
% estoppel or otherwise, any license or right under (i) any existing
% or later issuing patent, whether or not the use of information in
% this software necessarily employs an invention of any existing or
% later issued patent, (ii) any copyright, (iii) any trademark, or
% (iv) any other intellectual property right.         
%           
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS "AS IS" AND
% ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
% TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
% PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE    
% COPYRIGHT OWNER BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
% LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
% USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
% AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT  
% LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
% IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
% THE POSSIBILITY OF SUCH DAMAGE.
%
%-------------------------------------------------------------------------

switch nargin
    case 5
        direction = 'forward';
    case 4
        direction = 'forward';
        zero_pad=0;
    case {1,2,3}
        error('nrsh:num_rec:input_args', 'Error in nrsh: not enough input arguments.')
end

if strcmpi(direction, 'inverse')
    rec_dist = -rec_dist;
end

[Ny,Nx,~]=size(hol);
if zero_pad==1
    recons=zeros(Ny*2,Nx*2,'single');
    recons(Ny/2+1:Ny/2+Ny, Nx/2+1:Nx/2+Nx) = single(hol);
    Ny=Ny*2;
    Nx=Nx*2;
else
    recons = single(hol);
end

persistent F
if (isempty(F))
    dx = 1/(Nx*pitch);
    fx = single(-1/(2*pitch):dx:1/(2*pitch)-dx);
    if isequal(Nx,Ny)
        [F,~]=meshgrid(fx);
        F = ifftshift(F.^2+F'.^2);
    else
        dy = 1/(Ny*pitch);
        fy = single(-1/(2*pitch):dy:1/(2*pitch)-dy);
        [F,F_temp] = meshgrid(fx,fy);
        F = ifftshift(F.^2 + F_temp.^2);
        clearvars F_temp;
    end
end

H = exp(-2i*pi*sqrt(max(0,(1/wlen)^2-F))*rec_dist);
H(H == 1) = 0;

recons=fft2(recons);
recons=recons.*H;
recons=ifft2(recons);

if zero_pad==1
    Ny=Ny/2;
    Nx=Nx/2;
    recons=recons(Ny/2+1:Ny/2+Ny,Nx/2+1:Nx/2+Nx, :);
end

end
