function [recons] = rec_fresnel(hol, pitch, wlen, rec_dist, direction)
%REC_FRESNEL Frensnel Method implementation.
%
%   Inputs:
%    hol               - input hologram to reconstruct
%    pitch             - pixel pitch in meters
%    wlen              - wavelength in meters.
%    rec_dist          - reconstruction distance in meters
%    direction         - reconstruction direction. It should be one of
%                        the following char. arrays: forward (propagation
%                        towards the object plane) or inverse (propagation 
%                        towards the hologram plane)
%                       
%   Output:
%    recons            - reconstructed field (complex magnitude)
%
%-------------------------------------------------------------------------
% Copyright(c) 2019
% University of Cagliari
% Department of Electrical and Electronic Engineering
% Italy
% All Rights Reserved.
%-------------------------------------------------------------------------                                        
%                                                                      
% The University of Cagliari - Department of Electrical and Electronic
% Engineering hereby grants to ISO/IEC JTC1 SC29 WG1
% (JPEG Committee) and each Member of ISO/IEC JTC1 SC29 WG1 (JPEG
% Committee) who participate in the Working Group dedicated to the
% standardization of JPEG Pleno, a non-exclusive, nontransferable,  
% worldwide, license under "University of Cagliari - Department of
% Electrical and Electronic Engineering" copyrights
% in this software to reproduce, distribute, display, perform and 
% create derivative works for the sole and exclusive purposes of 
% creating a hologram reconstruction software in the frame of
% the JPEG Pleno standard.
%
% Modifications to this code shall be clearly indicated and      
% identified by the relevant copyright notice(s) of the party   
% generating these changes and/or derivative works.            
%   
% Nothing contained in this software shall, except as herein 
% expressly provided, be construed as conferring by implication,
% estoppel or otherwise, any license or right under (i) any existing
% or later issuing patent, whether or not the use of information in
% this software necessarily employs an invention of any existing or
% later issued patent, (ii) any copyright, (iii) any trademark, or
% (iv) any other intellectual property right.         
%           
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS "AS IS" AND
% ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
% TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
% PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE    
% COPYRIGHT OWNER BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
% LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
% USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
% AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT  
% LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
% IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
% THE POSSIBILITY OF SUCH DAMAGE.
%
%-------------------------------------------------------------------------

k=2*pi/wlen;
[rows,cols] = size(hol);

persistent F
if ( isempty(F) || (rows~=size(F,1)) ||  (cols~=size(F,2)) )
    Lx=cols*pitch;
    Ly=rows*pitch;
    
    if isequal(Lx,Ly)
        x=-Lx/2:pitch:Lx/2-pitch;
        [F,~]=meshgrid(x);
        F=F.^2+F'.^2;
    else
        x=-Lx/2:pitch:Lx/2-pitch;
        y=-Ly/2:pitch:Ly/2-pitch;

        [F,F_temp] = meshgrid(x,y);
        F=F.^2+F_temp.^2;
        clearvars F_temp
    end
end

if strcmpi(direction, 'forward')
    recons=((-1i/(wlen*rec_dist))*exp((1i*k/(2*rec_dist))*F)).*hol;
    recons=ifftshift(ifft2(recons));
else
    recons=fft2(fftshift(hol));
    recons=((-(wlen*rec_dist)/1i)*exp(-(1i*k/(2*rec_dist))*F)).*recons;
end 
